#!/usr/bin/env python3

# https://github.com/m-jowett/outcp

# The MIT License (MIT)
# Copyright (c) 2016 Matthew Jowett
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import http.client
import socket
import os
from . import __version__ as version

def test(port = -1, hostname = ""):
    if port == -1:
        print("Error: Port cannot be blank.", end="")
        return -1
    elif hostname == "":
        print("Error: Hostname cannot be blank.", end="")
        return -1

    if isinstance(port, int) == 0:
        print("Error: \'" + str(port) + "\' (Port) must be an integer.", end="")
        return -1
    elif isinstance(hostname, str) == 0:
        print("Error: \'" + str(hostname) + "\' (Hostname) must be a string.", end="")
        return -1

    if port > 65535 or port < 0:
        print("Error: \'" + str(port) + "\' (Port) must be 0-65535.", end="")
        return -1

    try:
        ipdest = socket.gethostbyname(hostname)
    except socket.error:
        print("Error: \'" + str(hostname) + "\' (Hostname) could not be resolved.", end="")
        return -1

    try:
        connection = http.client.HTTPConnection(ipdest, port)
        connection.timeout = 4
        connection.request("HEAD", "/") # path
        result = connection.getresponse().status
        connection.close()
    except Exception as e:
        print("Error: " + str(e).strip(), end="")
        return -1

    return result

def testui(portlist = [], namelist = [], hostname = "", format = 1):
    if isinstance(portlist, list) == 0:
        x = portlist
        portlist = []
        portlist.append(str(x))

    if len(portlist) == 0:
        print("Error: Port, you must choose a port (or portlist).")
        return

    if isinstance(namelist, list) == 0:
        x = namelist
        namelist = []
        namelist.append(str(x))

    if len(namelist) == 0:
        print("Error: Name, you must choose a name (or namelist).")
        return

    if isinstance(hostname, str) == 0:
        print("Error: \'" + str(hostname) + "\' (Hostname) must be a string.")
        return

    portlist = list(map(int, portlist))
    namelist = list(map(str, namelist))

    if hostname == "":
        print("Error: Hostname cannot be blank.")
        return

    message = ""

    if format == 1:
        # Dynamic table size
        # Maximum found length of service string.
        servlen = 0
        # Maximum found length of port integer.
        portlen = 0

        for i in portlist:
            if len(str(i)) > portlen:
                portlen = len(str(i))

        for i in namelist:
            if len(i) > servlen:
                servlen = len(i)
    else:
        # Fixed table spacing
        servlen = 32
        portlen = len("65535")

    codelen = len("599")
    # Maximum error message length.
    notelen = 45

    try:
        ipdest = socket.gethostbyname(hostname)
    except socket.error:
        print("Error: \'" + str(hostname) + "\' (Hostname) could not be resolved.")
        return

    print("Testing TCP ports @ " + hostname + " (" + ipdest + ")" + " via HTTP request: ")
    for p, i2 in enumerate(portlist):
        try:
            i = namelist[p]
        except IndexError:
            i = "-"
        # | Service name | TCP Port | Repsonse code | Code description |
        print(i, end="")
        print(" " * (servlen - len(i)), end="    ")

        print(i2, end="")
        print(" " * (portlen - len(str(i2))), end="    ")

        result = test(i2, hostname)

        if result != -1:
            print(result, end="")
        print(" " * (codelen - len(str(result))), end="    ")

        response = ""
        if result != -1:
            try:
                response = http.client.responses[200]
            except KeyError:
                reponse = "-"

            print(response, end="")
            print(" " * (notelen - len(response)), end="\n")
        else:
            print()

        if i2 == 80 and (response != "OK" or result == -1):
            message += "\nAccess to port 80 (HTTP) is normally not blocked. It is more likely that there is a network error or the host is down."

    print(message, end="")

def main():
    print("outcp v" + str(version))
    print("""Make sure that: This Python application is allowed through your PCs local firewall.
                The host allows connection under the port specified, otherwise it will return as blocked.\n""")

    inputlist = []
    for i in list(map(str, input("Enter the list of hostnames (seperated by a space)\n: ").split())):
        for ii in list(map(str, input("Enter the list of ports for " + i + "\n: ").split())):
            inputlist.append("n" + str(input("Enter the port name of " + ii + " for " + i + "\n: ")))
            inputlist.append("p" + ii)
        inputlist.append("h" + i)

    namelist = []
    portlist = []
    for i in inputlist:
        if i[:1] == "n":
            namelist.append(i[1:])
        elif i[:1] == "p":
            portlist.append(i[1:])
        elif i[:1] == "h":
            print()
            testui(portlist, namelist, i[1:], 1)
            namelist = []
            portlist = []

if __name__ == "__main__":
    main()
    exit()
